@@ -26,7 +26,7 @@
 #include "depmod.h"
 #include "logging.h"
 #include "index.h"
-#include "moduleops.h"
+#include "elfops.h"
 #include "tables.h"
 #include "config_filter.h"
 
@@ -276,37 +276,14 @@ static struct module *grab_module(const char *dirname, const char *filename)
 	INIT_LIST_HEAD(&new->dep_list);
 	new->order = INDEX_PRIORITY_MIN;
 
-	new->data = grab_file(new->pathname, &new->len);
-	if (!new->data) {
+	new->file = grab_elf_file(new->pathname);
+	if (!new->file) {
 		warn("Can't read module %s: %s\n",
 		     new->pathname, strerror(errno));
-		goto fail_data;
+		free(new);
+		return NULL;
 	}
-
-	switch (elf_ident(new->data, new->len, &new->conv)) {
-	case ELFCLASS32:
-		new->ops = &mod_ops32;
-		break;
-	case ELFCLASS64:
-		new->ops = &mod_ops64;
-		break;
-	case -ENOEXEC:
-		warn("Module %s is not an elf object\n", new->pathname);
-		goto fail;
-	case -EINVAL:
-		warn("Module %s has unknown endianness\n", new->pathname);
-		goto fail;
- 	default:
-		warn("Module %s has unknown word size\n", new->pathname);
- 		goto fail;
- 	}
 	return new;
-
-fail:
-	release_file(new->data, new->len);
-fail_data:
-	free(new);
-	return NULL;
 }
 
 struct module_traverse
@@ -667,17 +644,62 @@ static struct module *sort_modules(const char *dirname, struct module *list)
 	return tlist;
 }
 
+/* Calculate the dependencies for this module */
+static void calculate_deps(struct module *module)
+{
+	unsigned int i;
+	struct string_table *symnames;
+	struct string_table *symtypes;
+	struct elf_file *file;
+
+	module->num_deps = 0;
+	module->deps = NULL;
+	file = module->file;
+
+	symnames = file->ops->load_dep_syms(module->pathname, file, &symtypes);
+	if (!symnames || !symtypes)
+		return;
+
+	for (i = 0; i < symnames->cnt; i++) {
+		const char *name;
+		struct module *owner;
+		int weak;
+
+		name = symnames->str[i];
+		weak = (*(symtypes->str[i]) == 'W');
+		owner = find_symbol(name, module->pathname, weak);
+		if (owner) {
+			info("%s needs \"%s\": %s\n",
+			       module->pathname, name,
+			       owner->pathname);
+			add_dep(module, owner);
+		}
+	}
+
+	free(symnames);
+	free(symtypes);
+}
+
 static struct module *parse_modules(struct module *list)
 {
 	struct module *i;
+	struct elf_file *file;
+	struct string_table *syms;
+	int j;
 
 	for (i = list; i; i = i->next) {
-		i->ops->load_symbols(i);
-		i->ops->fetch_tables(i);
+		file = i->file;
+		syms = file->ops->load_symbols(file);
+		if (syms) {
+			for (j = 0; j < syms->cnt; j++)
+				add_symbol(syms->str[j], i);
+			strtbl_free(syms);
+		}
+		file->ops->fetch_tables(file, &i->tables);
 	}
 	
 	for (i = list; i; i = i->next)
-		i->ops->calculate_deps(i);
+		calculate_deps(i);
 	
 	/* Strip out modules with dependency loops. */
  again:
@@ -747,38 +769,42 @@ static void output_symbols_bin(struct module *unused, FILE *out, char *dirname)
 static void output_aliases(struct module *modules, FILE *out, char *dirname)
 {
 	struct module *i;
-	const char *p;
-	unsigned long size;
+	struct elf_file *file;
+	struct string_table *tbl;
+	int j;
 
 	fprintf(out, "# Aliases extracted from modules themselves.\n");
 	for (i = modules; i; i = i->next) {
 		char modname[strlen(i->pathname)+1];
 
+		file = i->file;
 		filename2modname(modname, i->pathname);
 
 		/* Grab from old-style .modalias section. */
-		for (p = i->ops->get_aliases(i, &size);
-		     p;
-		     p = next_string(p, &size))
-			fprintf(out, "alias %s %s\n", p, modname);
+		tbl = file->ops->load_strings(file, ".modalias", NULL, fatal);
+		for (j = 0; tbl && j < tbl->cnt; j++)
+			fprintf(out, "alias %s %s\n", tbl->str[j], modname);
+		strtbl_free(tbl);
 
-		/* Grab form new-style .modinfo section. */
-		for (p = i->ops->get_modinfo(i, &size);
-		     p;
-		     p = next_string(p, &size)) {
+		/* Grab from new-style .modinfo section. */
+		tbl = file->ops->load_strings(file, ".modinfo", NULL, fatal);
+		for (j = 0; tbl && j < tbl->cnt; j++) {
+			const char *p = tbl->str[j];
 			if (strstarts(p, "alias="))
 				fprintf(out, "alias %s %s\n",
 					p + strlen("alias="), modname);
 		}
+		strtbl_free(tbl);
 	}
 }
 
 static void output_aliases_bin(struct module *modules, FILE *out, char *dirname)
 {
 	struct module *i;
-	const char *p;
+	struct elf_file *file;
+	struct string_table *tbl;
+	int j;
 	char *alias;
-	unsigned long size;
 	struct index_node *index;
 	int duplicate;
 
@@ -787,13 +813,13 @@ static void output_aliases_bin(struct module *modules, FILE *out, char *dirname)
 	for (i = modules; i; i = i->next) {
 		char modname[strlen(i->pathname)+1];
 
+		file = i->file;
 		filename2modname(modname, i->pathname);
 
 		/* Grab from old-style .modalias section. */
-		for (p = i->ops->get_aliases(i, &size);
-		     p;
-		     p = next_string(p, &size)) {
-			alias = NOFAIL(strdup(p));
+		tbl = file->ops->load_strings(file, ".modalias", NULL, fatal);
+		for (j = 0; tbl && j < tbl->cnt; j++) {
+			alias = NOFAIL(strdup(tbl->str[j]));
 			underscores(alias);
 			duplicate = index_insert(index, alias, modname, i->order);
 			if (duplicate && warn_dups)
@@ -801,11 +827,12 @@ static void output_aliases_bin(struct module *modules, FILE *out, char *dirname)
 					alias, modname);
 			free(alias);
 		}
+		strtbl_free(tbl);
 
 		/* Grab from new-style .modinfo section. */
-		for (p = i->ops->get_modinfo(i, &size);
-		     p;
-		     p = next_string(p, &size)) {
+		tbl = file->ops->load_strings(file, ".modinfo", NULL, fatal);
+		for (j = 0; tbl && j < tbl->cnt; j++) {
+			const char *p = tbl->str[j];
 			if (strstarts(p, "alias=")) {
 				alias = NOFAIL(strdup(p + strlen("alias=")));
 				underscores(alias);
@@ -816,6 +843,7 @@ static void output_aliases_bin(struct module *modules, FILE *out, char *dirname)
 				free(alias);
 			}
 		}
+		strtbl_free(tbl);
 	}
 	
 	index_write(index, out);
